<?php
/*
Plugin Name: Zaantar's Subscribe To Comments
Version: 1
Description: Umožňuje (prozatím jen přihlášeným) čtenářům snadno přijímat upozornění na nové komentáře u vybraných článků. Inspirováno pluginem <a href="http://txfx.net/code/wordpress/subscribe-to-comments/">Subscribe To Comments</a>. <strong>Vyvinuto a určeno pro blogosphere.cz.</strong> 
Author: Zaantar
Author URI: http://zaantar.eu
*/

/*****************************************************************************\
		DATABASE
\*****************************************************************************/

register_activation_hook( __FILE__, 'stcr_plugin_activation' );

function stcr_plugin_activation() {
	// vyrobime tabulky na blogu
	global $wpdb;
	$subscriptions_structure = 
        'CREATE TABLE IF NOT EXISTS '.$wpdb->prefix.'stcr_subscriptions (
            id INT NOT NULL AUTO_INCREMENT,
            user_id BIGINT(20),
            post_id BIGINT(20), 
            UNIQUE ( id ),
            PRIMARY KEY ( id )
        )';
        
    $wpdb->query( $wpdb->prepare( $subscriptions_structure ) );
}


function stcr_is_subscribed( $post_id ) {
	global $wpdb;
	$query = 'SELECT COUNT(*)
		FROM '.$wpdb->prefix.'stcr_subscriptions
		WHERE ( post_id = '.$post_id.'
			AND user_id = '.get_current_user_id().' )';
	$count = $wpdb->get_var( $wpdb->prepare( $query ) );
	return ( $count > 0 );
}


function stcr_perform_action( $action, $post_id ) {
	//echo '<br>action='.$action.', post_id='.$post_id.'<br>';
	switch( $action ) {
	case 'subscribe':
		return stcr_subscribe( $post_id );
		break;
	case 'unsubscribe':
		return stcr_unsubscribe( $post_id );
		break;
	case 'choice':
		if( isset( $_REQUEST['stcr_choice'] ) ) {
			return stcr_subscribe( $post_id );
		} else {
			return true;
		}
		break;
	case 'none':
		return true;
		break;
	default:
		return false;
		break;
	}
}


function stcr_subscribe( $post_id ) {
	global $wpdb;
	if( stcr_is_subscribed( $post_id ) ) {
		return true;
	} else {
		return $wpdb->insert( $wpdb->prefix.'stcr_subscriptions', 
			array( 'post_id' => $post_id, 'user_id' => get_current_user_id() ), array( '%d', '%d' ) );
	}
}


function stcr_unsubscribe( $post_id ) {
	global $wpdb;
	if( !stcr_is_subscribed( $post_id ) ) {
		return true;
	} else {
		$query = 'DELETE FROM '.$wpdb->prefix.'stcr_subscriptions
			WHERE ( user_id = '.get_current_user_id().'
				AND post_id = '.$post_id.' )';
		return $wpdb->query( $wpdb->prepare( $query ) );
	}
}	


function stcr_get_subscribers( $post_id ) {
	global $wpdb;
	$query = 'SELECT user_id
		FROM '.$wpdb->prefix.'stcr_subscriptions
		WHERE post_id = '.$post_id;
	$uids = $wpdb->get_col( $wpdb->prepare( $query ) );
	return $uids;
}


function stcr_get_subscriber_emails( $post_id ) {
	$uids = stcr_get_subscribers( $post_id );
	$results = array();
	foreach( $uids as $uid ) {
		$userdata = get_userdata( $uid );
		$results[$userdata->user_email] = $userdata->user_email;
	}
	return $results;
}


function stcr_get_user_subscriptions( $user_id ) {
	global $wpdb;
	$query = 'SELECT post_id
		FROM '.$wpdb->prefix.'stcr_subscriptions
		WHERE user_id='.$user_id;
	$pids = $wpdb->get_col( $wpdb->prepare( $query ) );
	return $pids;
};


/*****************************************************************************\
		USER DEFAULT BEHAVIOUR SETTINGS
\*****************************************************************************/

add_action( 'show_user_profile','stcr_show_user_profile' );
add_action( 'edit_user_profile','stcr_show_user_profile' );
add_action( 'personal_options_update', 'stcr_edit_user_profile' );
add_action( 'edit_user_profile_update', 'stcr_edit_user_profile' );

function stcr_show_user_profile( $user ) {
	$setting = get_user_meta( $user->ID, 'stcr_default', true );
	if( $setting == '' ) {
		$setting = 'choice';
	}
	?>
	<h3>Upozorňování na komentáře e-mailem</h3>
	<p>Zde si můžete nastavit, zda chcete být upozorňováni na další komentáře u článků, které budete vy komentovat.</p>
	<table class="form-table">
		<tr>
		    <th><label for="stcr_default">Výchozí nastavení</label></th>
		    <td>
		    	<select name="stcr_default">
		    		<option value="never" <?php if( $setting == 'never' ) echo 'selected="selected"'; ?> >
		    			Nikdy neupozorňovat
		    		</option>
		    		<option value="choice" <?php if( $setting == 'choice' ) echo 'selected="selected"'; ?> >
		    			Zobrazit volbu
		    		</option>
		    		<option value="always" <?php if( $setting == 'always' ) echo 'selected="selected"'; ?> >
		    			Vždy upozorňovat
		    		</option>
		    	</select>
		    </td>
		</tr>
	</table>
	<p>Toto nastavení již existující nastavení u jednotlivých (vámi dříve komentovaných) článků.</p>
	<?php
}


function stcr_edit_user_profile( $user_id ) {
	if ( !current_user_can( 'edit_user', $user_id ) ) {
		return false; 
	}
	update_user_meta( $user_id, 'stcr_default', $_POST['stcr_default'] );
}



/*****************************************************************************\
		COMMENT FORM
\*****************************************************************************/

add_action( 'comment_form', 'stcr_comment_form' );

function stcr_comment_form( $post_id = '0' ) {

	if( !is_user_logged_in() ) {
		return; // TODO
	}
	
	// zpracujeme případné akce
	if( isset($_REQUEST['stcr_action']) ) {
        $action = $_REQUEST['stcr_action'];
    } else {
        $action = 'none';
    }

    stcr_perform_action( $action, $post_id );
    
    // zobrazíme příslušné možnosti
	if( !stcr_is_post_author( $post_id )  && !stcr_is_subscribed( $post_id ) ) {
		// pokud není čtenář autorem a není přihlášen k odběru, budeme se řídit jeho výchozím nastavením.
		$default = get_user_meta( get_current_user_id(), 'stcr_default', true );
		//$profile_url = 
		switch( $default ) {
			case 'always':
				?>
				<input type="hidden" name="stcr_action" value="subscribe" />
				<p><small>Upozornění na nové komentáře k tomuto článku vám budou zaslána e-mailem. Chcete-li, můžete si nastavení změnit <a href="/wp-admin/profile.php">ve svém profilu</a>.</small></p>
				<?php
				break;
			case 'never':
				?>
				<input type="hidden" name="stcr_action" value="none" />
				<p><small><a href="/wp-admin/profile.php">Ve svém profilu</a> si můžete nastavit zasílání upozornění na nové komentáře e-mailem jako výchozí volbu.</small></p>
				<?php
				break;
			case 'choice':
			default:
				?>
				<input type="hidden" name="stcr_action" value="choice" />
				<p>
					<input type="checkbox" name="stcr_choice" checked="checked" />
					<label for="stcr_choice">Zasílat upozornění na nové komentáře e-mailem 
						<small>(výchozí nastavení můžete upravit <a href="/wp-admin/profile.php">ve svém profilu</a>)</small>.
					</label>
				</p>
				<?php
				break;
		
		}
	}
}

add_action( 'comment_form_after', 'stcr_comment_form_after' );

function stcr_comment_form_after() {
	global $post;
	
	if( !is_user_logged_in() ) {
		return; // TODO
	}

	if( stcr_is_post_author( $post->ID ) ) {
		?>
		<p><small>Jste autorem tohoto článku. <del>Spravovat odběry upozornění</del>.</small></p>
		<?php
	} elseif( stcr_is_subscribed( $post->ID ) ) {
		?>
		<p>
			<form method="post" action="<?php echo get_permalink(); ?>">
				<input type="hidden" name="stcr_action" value="unsubscribe">
				<label for="submit">Momentálně jste přihlášeni k odběru upozornění na nové komentáře u tohoto článku.</label>
				<input type="submit" name="submit" value="Odhlásit" />
			</form>
		</p>
		<?php
	} else {
		?>
		<p>
			<form method="post" action="<?php echo get_permalink(); ?>">
				<input type="hidden" name="stcr_action" value="subscribe">
				<input type="submit" name="submit" value="Přihlásit k odběru upozornění na nové komentáře" /><br />
				<small><label for="submit">(lze provést, aniž byste sami komentovali)</label></small>
			</form>
		</p>
		<?php
	}
	?>
	<p>Doplněk pro odběr upozornění je stále ve vývoji. Pokud při jeho používání narazíte na jakoukoliv chybu nebo neočekávané chování, pište, prosím, na <a href="mailto:admin@blogosphere.cz">admin@blogosphere.cz</a>. Děkuji. </p>
	<?php	
}


function stcr_is_post_author( $post_id ) {
	$post = get_post( $post_id );
	return ( $post->post_author == get_current_user_id() );
}



/*****************************************************************************\
		NEW COMMENT
\*****************************************************************************/


add_action( 'comment_post', 'stcr_new_comment' );

function stcr_new_comment( $comment_id ) {
	if( !is_user_logged_in() ) {
		return; // TODO
	}

	$comment = get_comment( $comment_id );
	
	// zpracujeme případné akce (subscribe/unsubscribe)
	if( isset($_REQUEST['stcr_action']) ) {
        $action = $_REQUEST['stcr_action'];
    } else {
        $action = 'none';
    }

    stcr_perform_action( $action, $comment->comment_post_ID );
	
	// rozešleme upozornění pouze pro schválené komentáře, které nejsou pingback ani trackback 
	if( $comment->comment_approved == '1' && $comment->comment_type == '' ) {
	
		$addresses = stcr_get_subscriber_emails( $comment->comment_post_ID );
		$subject = '[%BLOG_NAME%] Nový komentář u článku: %POST_TITLE%';
		$message = 

'U článku "%POST_TITLE%" je nový komentář.
%COMMENT_URL%

Autor: %COMMENT_AUTHOR%
Text komentáře:
%COMMENT_TEXT%

Všechny komentáře u článku naleznete zde:
%POST_COMMENT_URL%

Pokud si nepřejete dostávat upozornění na další komentáře, upravte si, prosím, nastavení u příslušného článku (možnost odhlásit odběr se nachází pod formulářem pro psaní nových komentářů). Narazíte-li na jakýkoliv problém nebo nejasnost, kontaktujte administrátora.';

		$subject = stcr_parse( $subject, $comment_id );
		$message = stcr_parse( $message, $comment_id );
		wp_mail( $addresses, $subject, $message );
	}
}


function stcr_parse( $text, $comment_id ) {

	$comment = get_comment( $comment_id );
	$post = get_post( $comment->comment_post_ID );

	$patterns[0] = '/%BLOG_NAME%/';
	$replacements[0] = get_bloginfo( 'name' );
	
	$patterns[1] = '/%POST_TITLE%/';
	$replacements[1] = $post->post_title;
	
	$patterns[2] = '/%POST_COMMENT_URL%/';
	$replacements[2] = get_permalink( $comment->comment_post_ID ).'#comments';
	
	$patterns[3] = '/%COMMENT_AUTHOR%/';
	$replacements[3] = $comment->comment_author;
	
	$patterns[4] = '/%COMMENT_URL%/';
	$replacements[4] = get_permalink( $comment->comment_post_ID ).'#comment-'.$comment_id;
	
	$patterns[5] = '/%COMMENT_TEXT%/';
	$replacements[5] = $comment->comment_content;
	
	$result = preg_replace( $patterns, $replacements, $text );
	return $result;
}


/*****************************************************************************\
		ADMIN SUBSCRIPTION OVERVIEW
\*****************************************************************************/

add_action( 'admin_menu','stcr_admin_subscription_overview' );

function stcr_admin_subscription_overview() {
	add_submenu_page( 'users.php', 'Uživatelé odebírající upozornění na komentáře', 'Uživatelé odebírající upozornění na komentáře', 'edit_posts', 
		'stcr-admin-subscription-overview', 'stcr_admin_subscription_overview_page' );
}


function stcr_admin_subscription_overview_page() {
	if( isset($_REQUEST['action']) ) {
        $action = $_REQUEST['action'];
    } else {
        $action = 'default';
    }
    
    switch( $action ) {
	case 'default':
	default:
		stcr_admin_subscription_overview_default();
	}
}


function stcr_admin_subscription_overview_default() {
	if( isset( $_REQUEST['stcr_page'] ) ) {
		$page = $_REQUEST['stcr_page'];
	} else {
		$page = 1;
	}
	if( current_user_can( 'manage_options' ) ) {
		$posts = get_posts( array( 
			'posts_per_page' => 50,
			'paged' => $page
		) ); 
	} else {
		$posts = get_posts( array(
			'author' => get_current_user_id(), 
			'posts_per_page' => 50,
			'paged' => $page
		) );
	}
	?>
	<div id="wrap">
		<h2>Uživatelé odebírající upozornění na komentáře</h2>
		<p>Pokud nemáte na tomto blogu administrátorská práva, uvidíte zde informace pouze pro články, jejichž jste autorem.</p>
		<p>
			<?php
				if( $page > 1 ) {
					 echo '&laquo; <a href="users.php?page=stcr-admin-subscription-overview&stcr_page='.($page-1).'" >Předchozí stránka</a>&nbsp;';
				}
				if( count( $posts ) > 0 ) {
					echo '&nbsp;<a href="users.php?page=stcr-admin-subscription-overview&stcr_page='.($page+1).'" >Následující stránka</a>&raquo;';
				}
			?>
		</p>
		<table class="wp-list-table fixed widefat" cellspacing="0">
		    <thead>
		        <tr>
		            <th scope="col" class="manage-column">post_id, Článek</th>
		            <th scope="col" class="manage-column">Seznam odběratelů</th>
		        </tr>
			</thead>
			<tfoot>
		        <tr>
		            <th scope="col" class="manage-column">post_id, Článek</th>
		            <th scope="col" class="manage-column">Seznam odběratelů</th>
		        </tr>
			</tfoot>
			<?php 
				foreach( $posts as $post ) {
					$subscribers = stcr_get_subscribers( $post->ID );
					?>
					<tr>
						<td>
							<?php echo $post->ID; ?>: <?php echo '<a href="'.get_permalink( $post->ID ).'" >'.$post->post_title.'</a>'; ?>
						</td>
						<td>
							<?php
								if( $subscribers == NULL ) {
									echo 'Žádní odběratelé';
								}
								foreach( $subscribers as $subscriber ) {
									$userdata = get_userdata( $subscriber );
									echo $userdata->user_login.'; ';
								}
							?>
						</td>
					</tr>
					<?php
				}
			?>
		</table>
	</div>
	<?php
}


/*****************************************************************************\
		USER SUBSCRIPTION OVERVIEW
\*****************************************************************************/


add_action( 'admin_menu','stcr_user_subscription_overview' );

function stcr_user_subscription_overview() {
	add_submenu_page( 'users.php', 'Odběr upozornění na komentáře', 'Odběr upozornění na komentáře', 'read', 
		'stcr-user-subscription-overview', 'stcr_user_subscription_overview_page' );
}


function stcr_user_subscription_overview_page() {
	if( isset($_REQUEST['action']) ) {
        $action = $_REQUEST['action'];
    } else {
        $action = 'default';
    }
    
    switch( $action ) {
	case 'default':
	default:
		stcr_user_subscription_overview_default();
	}
}


function stcr_user_subscription_overview_default() {
	$subscriptions = stcr_get_user_subscriptions( get_current_user_id() );
	?>
	<div id="wrap">
		<h2>Přehled odběrů upozornění na komentáře</h2>
		<p>Na této stránce uvidíte seznam článků, od nichž odebíráte upozornění e-mailem na nové komentáře. Chcete-li odběr zrušit, učiňte tak přímo na stránce článku pod formulářem pro psaní komentářů.</p>
		<table class="wp-list-table fixed widefat" cellspacing="0">
		    <thead>
		        <tr>
		            <th scope="col" class="manage-column">post_id, Článek</th>
		        </tr>
			</thead>
			<tfoot>
		        <tr>
		            <th scope="col" class="manage-column">post_id, Článek</th>
		        </tr>
			</tfoot>
			<?php
				foreach( $subscriptions as $subscription ) {
					$post = get_post( $subscription );
					?>
					<tr>
						<td>
							<?php echo $post->ID; ?>: <?php echo '<a href="'.get_permalink( $post->ID ).'" >'.$post->post_title.'</a>'; ?>
						</td>
					</tr>
					<?php
				}
			?>
		</table>
	</div>
	<?php
}



?>
